@@ -4,6 +4,8 @@ import os
 import tempfile
 from datetime import date
 import pandas as pd
+from pandas.api.types import is_numeric_dtype
+from pandas.api.types import is_datetime64_any_dtype as is_datetime
 from numpy import nan
 import requests
 import urllib
@@ -104,6 +106,9 @@ class Data_Loader(ABC):
     get_years(nrows=1)
         Get years contained in data set
     """
+
+    _last_count = None
+
     @abstractmethod
     def get_count(self, year=None, *, agency=None, force=False, opt_filter=None, where=None):
         pass
@@ -112,7 +117,7 @@ class Data_Loader(ABC):
     def load(self, year=None, nrows=None, offset=0, *, pbar=True, agency=None, opt_filter=None, select=None, output_type=None):
         pass
 
-    def get_years(self, *, nrows=1, **kwargs):
+    def get_years(self, *, nrows=1, check=None, **kwargs):
         '''Get years contained in data set
         
         Parameters
@@ -129,7 +134,17 @@ class Data_Loader(ABC):
         if self.date_field==None:
             raise ValueError("A date field is required to get years")
 
-        year = date.today().year
+        check_input = check is not None
+
+        if check_input and len(check)==0:
+            return []
+
+        if check_input:
+            check = check.copy()
+            check.sort(reverse=True)
+            year = check.pop(0)
+        else:
+            year = date.today().year
 
         oldest_recent = 20
         max_misses_gap = 10
@@ -149,6 +164,12 @@ class Data_Loader(ABC):
             sleep(sleep_time)
 
             year-=1
+            if check_input:
+                if len(check)==0:
+                    break
+                while year not in check:
+                    year-=1
+                check.remove(year)
 
         return years
 
@@ -212,7 +233,9 @@ class Csv(Data_Loader):
             Record count or number of rows in data request
         '''
 
-        if ".zip" not in self.url and year==None and agency==None:            
+        if self._last_count is not None and self._last_count[0] == (self.url, year, agency):
+            return self._last_count[1]
+        if ".zip" not in self.url and year==None and agency==None:
             count = 0
             with requests.get(self.url, stream=True) as r:
                 for chunk in r.iter_content(chunk_size=2**16):
@@ -225,14 +248,15 @@ class Csv(Data_Loader):
                     count-=1
                 else:
                     break
-
-            return count
         elif force:
-            return len(self.load(year=year, agency=agency))
+            count = len(self.load(year=year, agency=agency))
         else:
             raise ValueError("Extracting the number of records for a single year of a CSV file requires reading the whole file in. In most cases, "+
                 "running load() with a year argument to load in the data and manually finding the record count will be more "
                 "efficient. If running get_count with a year argument is still desired, set force=True")
+        
+        self._last_count = ((self.url, year, agency), count)
+        return count
 
 
     def load(self, year=None, nrows=None, offset=0, *, pbar=True, agency=None, **kwargs):
@@ -268,17 +292,37 @@ class Csv(Data_Loader):
                 except Exception as e:
                     raise e
         else:
-            r = requests.head(self.url)
-            if r.status_code==404:
-                # Try get instead
-                r = requests.get(self.url)
+            use_legacy = False
             try:
-                r.raise_for_status()
-            except requests.exceptions.HTTPError as e:
-                raise OPD_DataUnavailableError(*e.args, _url_error_msg.format(self.url))
+                r = requests.head(self.url)
+            except requests.exceptions.SSLError as e:
+                if "[SSL: UNSAFE_LEGACY_RENEGOTIATION_DISABLED] unsafe legacy renegotiation disabled" in str(e.args[0]) or \
+                    "[SSL: CERTIFICATE_VERIFY_FAILED] certificate verify failed: unable to get local issuer certificate" in str(e.args[0]):
+                    use_legacy = True
+                else:
+                    raise e
             except Exception as e:
-                raise e
-            with requests.get(self.url, params=None, stream=True) as resp:
+                raise
+                
+            if not use_legacy:
+                if r.status_code==404:
+                    # Try get instead
+                    r = requests.get(self.url)
+                try:
+                    r.raise_for_status()
+                    r.close()
+                except requests.exceptions.HTTPError as e:
+                    raise OPD_DataUnavailableError(*e.args, _url_error_msg.format(self.url))
+                except Exception as e:
+                    raise e
+            
+            def get(url, use_legacy):
+                if use_legacy:
+                    return get_legacy_session().get(url, params=None, stream=True)
+                else:
+                    return requests.get(url, params=None, stream=True)
+                
+            with get(self.url, use_legacy) as resp:
                 try:
                     table = pd.read_csv(TqdmReader(resp, pbar=pbar), nrows=offset+nrows if nrows is not None else None, encoding_errors='surrogateescape')
                 except (urllib.error.HTTPError, pd.errors.ParserError) as e:
@@ -319,7 +363,8 @@ class Csv(Data_Loader):
             if self.date_field==None:
                 raise ValueError("No date field provided to access year information")
             df = self.load()
-            years = list(df[self.date_field].dt.year.dropna().unique())
+            date_col = pd.to_datetime(df[self.date_field])
+            years = list(date_col.dt.year.dropna().unique())
             years.sort()
             return [int(x) for x in years]
 
@@ -369,7 +414,7 @@ class Excel(Data_Loader):
         try:
             self.excel_file = pd.ExcelFile(url)
         except urllib.error.HTTPError as e:
-            if str(e) == "HTTP Error 406: Not Acceptable":
+            if str(e) in ["HTTP Error 406: Not Acceptable", 'HTTP Error 403: Forbidden']:
                 # 406 error: https://stackoverflow.com/questions/34832970/http-error-406-not-acceptable-python-urllib2
                 # File-like input for URL: https://stackoverflow.com/questions/57815780/how-can-i-directly-handle-excel-file-link-python/57815864#57815864
                 headers = {'User-agent' : 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_3) AppleWebKit/537.75.14 (KHTML, like Gecko) Version/7.0.3 Safari/7046A194A'}
@@ -429,8 +474,12 @@ class Excel(Data_Loader):
             Record count or number of rows in data request
         '''
 
-        if force:
-            return len(self.load(year=year, agency=agency))
+        if self._last_count is not None and self._last_count[0]==(self.url, year, agency):
+            return self._last_count[1]
+        elif force:
+            count = len(self.load(year=year, agency=agency))
+            self._last_count = ((self.url, year, agency), count)
+            return count
         else:
             raise ValueError("Extracting the number of records for an Excel file requires reading the whole file in. In most cases, "+
                 "running load() to load in the data and manually finding the record count will be more "
@@ -579,6 +628,16 @@ class Excel(Data_Loader):
         if nrows is not None and len(table)>nrows:
             table = table.head(nrows)
 
+        # Check for empty rows at the bottom
+        num_empty = table.isnull().sum(axis=1)
+        empty_rows = num_empty==len(table.columns)
+        if empty_rows.any():
+            # Check if all rows after first empty row are empty or almost empty
+            empty_rows = empty_rows[empty_rows]
+            num_empty = num_empty.loc[empty_rows.index[0]:]
+            if ((num_empty / len(table.columns)) > 0.75).all():
+                table = table.head(empty_rows.index[0])
+
         # Clean up column names
         table.columns = [x.strip() if isinstance(x, str) else x for x in table.columns]
 
@@ -633,14 +692,19 @@ class Excel(Data_Loader):
             years.sort()
             return list(year_dict.keys())
         if not force:
-            raise ValueError("Extracting the years of a CSV file requires reading the whole file in. In most cases, "+
+            raise ValueError("Extracting the years of a Excel file requires reading the whole file in. In most cases, "+
                 "running load() with no arguments to load in the whole CSV file and manually finding the years will be more "
                 "efficient. If running get_years is still desired, set force=True")
         else:
             if self.date_field==None:
                 raise ValueError("No date field provided to access year information")
             df = self.load()
-            years = list(df[self.date_field].dt.year.dropna().unique())
+            if is_datetime(df[self.date_field]):
+                years = list(df[self.date_field].dt.year.dropna().unique())
+            elif is_numeric_dtype(df[self.date_field]):
+                years = list(df[self.date_field].dropna().unique())
+            else:
+                raise TypeError("Unknown date column format")
             return [int(x) for x in years]
 
 
@@ -781,9 +845,13 @@ class Arcgis(Data_Loader):
             Record count or number of rows in data request
         '''
         
-        if where==None:
-            where, record_count = self.__construct_where(year)
+        if self._last_count is not None and self._last_count[0]==(year,where):
+            record_count = self._last_count[1]
+            where_query = self._last_count[2]
+        elif where==None:
+            where_query, record_count = self.__construct_where(year)
         else:
+            where_query = where
             try:
                 record_count = self.__request(where=where, return_count=True)["count"]
                 if self.verify:
@@ -798,6 +866,8 @@ class Arcgis(Data_Loader):
             except:
                 raise
 
+        self._last_count = ((year,where), record_count, where_query)
+
         return record_count
 
 
@@ -851,7 +921,10 @@ class Arcgis(Data_Loader):
 
     def __construct_where(self, year=None):
         where_query = ""
-        if self.date_field!=None and year!=None:
+        if self._last_count is not None and self._last_count[0]==(year,None):
+            record_count = self._last_count[1]
+            where_query = self._last_count[2]
+        elif self.date_field!=None and year!=None:
             where_query, record_count = self._build_date_query(year)
         else:
             where_query = '1=1'
@@ -869,6 +942,8 @@ class Arcgis(Data_Loader):
             except:
                 raise
 
+        self._last_count = ((year,None), record_count, where_query)
+
         return where_query, record_count
 
     
@@ -887,7 +962,7 @@ class Arcgis(Data_Loader):
                 if k==0:
                     where_query = f"{self.date_field} >= '{start_date}' AND  {self.date_field} < '{stop_date}'"
                 else:
-                    break
+                    # break
                     # Dataset (San Jose crash data) that required this does not function well so removing its functionality for now to speed up this function.
                     # This is the recommended way but it has been found to not work sometimes. One dataset was found that requires this.
                     # https://gis.stackexchange.com/questions/451107/arcgis-rest-api-unable-to-complete-operation-on-esrifieldtypedate-in-query
@@ -1084,7 +1159,7 @@ class Arcgis(Data_Loader):
 
         df = pd.DataFrame.from_records([x["attributes"] for x in features])
         for col in date_cols:
-            df[col] = pd.to_datetime(df[col], unit="ms")
+            df[col] = pd.to_datetime(df[col], unit="ms", errors='coerce')
 
         if len(df) > 0:
             has_point_geometry = any("geometry" in x and "x" in x["geometry"] for x in features)
@@ -1119,8 +1194,6 @@ class Arcgis(Data_Loader):
 
                     if "geolocation" not in df:
                         df["geolocation"] = geometry
-                    else:
-                        raise KeyError("geolocation already exists in DataFrame")
 
             return df
         else:
@@ -1194,11 +1267,17 @@ class Carto(Data_Loader):
         int
             Record count or number of rows in data request
         '''
-        
-        where = self.__construct_where(year)
-        json = self.__request(where=where, return_count=True)
 
-        return json["rows"][0]["count"]
+        if self._last_count is not None and self._last_count[0]==year:
+            return self._last_count[1]
+        else:
+            where = self.__construct_where(year)
+            json = self.__request(where=where, return_count=True)
+            count = json["rows"][0]["count"]
+
+        self._last_count = (year, count, where)
+
+        return count
 
 
     def __request(self, where=None, return_count=False, out_fields="*", out_type="GeoJSON", offset=0, count=None):
@@ -1270,9 +1349,14 @@ class Carto(Data_Loader):
             DataFrame containing downloaded
         '''
         
-        where_query = self.__construct_where(year)
-        json = self.__request(where=where_query, return_count=True)
-        record_count = json["rows"][0]["count"]
+        if self._last_count is not None and self._last_count[0]==year:
+            record_count = self._last_count[1]
+            where_query = self._last_count[2]
+        else:
+            where_query = self.__construct_where(year)
+            json = self.__request(where=where_query, return_count=True)
+            record_count = json["rows"][0]["count"]
+            self._last_count = (year, record_count, where_query)
 
         record_count-=offset
         if record_count<=0:
@@ -1334,7 +1418,7 @@ class Carto(Data_Loader):
                 if use_gpd:
                     geometry = []
                     for feat in features:
-                        if "geometry" not in feat or feat["geometry"]==None:
+                        if "geometry" not in feat or feat["geometry"]==None or len(feat["geometry"]["coordinates"])<2:
                             geometry.append(None)
                         else:
                             geometry.append(Point(feat["geometry"]["coordinates"][0], feat["geometry"]["coordinates"][1]))
@@ -1440,14 +1524,30 @@ class Socrata(Data_Loader):
 
         if where==None:
             where = self.__construct_where(year, opt_filter)
+
+        if self._last_count is not None and self._last_count[0]==(year, opt_filter, where):
+            return self._last_count[1]
+
+        try:
+            results = self.client.get(self.data_set, where=where, select="count(*)")
+        except (requests.HTTPError, requests.exceptions.ReadTimeout) as e:
+            raise OPD_SocrataHTTPError(self.url, self.data_set, *e.args, _url_error_msg.format(self.url))
+        except Exception as e: 
+            if len(e.args)>0 and (e.args[0]=='Unknown response format: text/html' or \
+                "Read timed out" in e.args[0]):
+                raise OPD_SocrataHTTPError(self.url, self.data_set, *e.args, _url_error_msg.format(self.url))
+            else:
+                raise e  
             
-        results = self.client.get(self.data_set, where=where, select="count(*)")
         try:
             num_rows = float(results[0]["count"])
         except:
             num_rows = float(results[0]["count_1"]) # Value used in VT Shootings data
 
-        return int(num_rows)
+        count = int(num_rows)
+        self._last_count = ((year, opt_filter, where),count)
+
+        return count
 
 
     def load(self, year=None, nrows=None, offset=0, *, pbar=True, opt_filter=None, select=None, output_type=None, **kwargs):
@@ -1652,13 +1752,15 @@ def filter_dataframe(df, date_field=None, year_filter=None, agency_field=None, a
             # In a future version, `df.iloc[:, i] = newvals` will attempt to set the values inplace instead of always setting a new array. 
             # To retain the old behavior, use either `df[df.columns[i]] = newvals` or, if columns are non-unique, `df.isetitem(i, newvals)`
             warnings.simplefilter("ignore", category=FutureWarning)
-            df.loc[:, date_field] = pd.to_datetime(df[date_field])
+            if date_field.lower()!="year":
+                df.loc[:, date_field] = pd.to_datetime(df[date_field])
     
     if year_filter != None and date_field != None:
         if isinstance(year_filter, list):
             df = df[(df[date_field].dt.year >= year_filter[0]) & (df[date_field].dt.year <= year_filter[1])]
         else:
-            df = df[df[date_field].dt.year == year_filter]
+            date_col = pd.to_datetime(df[date_field])
+            df = df[date_col.dt.year == year_filter]
 
     if agency != None and agency_field != None:
         df = df.query(agency_field + " = '" + agency + "'")
@@ -1710,16 +1812,4 @@ class TqdmReader:
         except StopIteration:
             if self.pbar:
                 self.bar.update(self.bar.total - self.bar.n)
-            return ""
-
-
-if __name__ == "__main__":
-    import time
-    _default_limit = 10000
-    start_time = time.time()
-    url = "https://gis.charlottenc.gov/arcgis/rest/services/CMPD/CMPD/MapServer/13/"
-    # url = 'https://services1.arcgis.com/zdB7qR0BtYrg0Xpl/arcgis/rest/services/ODC_CRIME_STOPS_P/FeatureServer/32'
-    date_field = 'YR'
-    years = _get_years("ArcGIS", url, date_field)
-    load_arcgis(url, date_field, [2020,2021])
-    print(f"Completed in {time.time()-start_time} seconds")
+            return ""
\ No newline at end of file
