@@ -50,15 +50,17 @@ class TestProduct:
         r.raise_for_status()
         assert count==r.json()["rows"][0]["count"]
 
-        df = loader.load(year=year)
+        df = loader.load(year=year, pbar=False)
+
+        assert len(df)==count
 
         offset = 1
         nrows = count - 2
-        df_offset = loader.load(year=year, nrows=nrows, offset=1)
+        df_offset = loader.load(year=year, nrows=nrows, offset=1, pbar=False)
 
         assert df_offset.equals(df.iloc[offset:offset+nrows].reset_index(drop=True))
 
-        df_offset = loader.load(year=year, offset=1)
+        df_offset = loader.load(year=year, offset=1, pbar=False)
         assert df_offset.equals(df.iloc[offset:].reset_index(drop=True))
 
         r = requests.get(f"https://phl.carto.com/api/v2/sql?format=GeoJSON&q=SELECT * FROM {dataset} WHERE {date_field} >= '{year}-01-01' AND {date_field} < '{year+1}-01-01'")
@@ -71,7 +73,7 @@ class TestProduct:
             from shapely.geometry import Point
             geometry = []
             for feat in features:
-                if "geometry" not in feat or feat["geometry"]==None:
+                if "geometry" not in feat or feat["geometry"]==None or len(feat["geometry"]["coordinates"])<2:
                     geometry.append(None)
                 else:
                     geometry.append(Point(feat["geometry"]["coordinates"][0], feat["geometry"]["coordinates"][1]))
@@ -88,7 +90,7 @@ class TestProduct:
         if data_loaders._has_gpd:
             assert type(df) == gpd.GeoDataFrame
             data_loaders._has_gpd = False
-            df = loader.load(year=year, nrows=nrows)
+            df = loader.load(year=year, nrows=nrows, pbar=False)
             data_loaders._has_gpd = True
             assert isinstance(df, pd.DataFrame)
 
@@ -123,6 +125,8 @@ class TestProduct:
         df = gis.load()
         count = gis.get_count()
 
+        assert len(df)==count
+
         offset = 1
         nrows = count-offset
         df_offset = gis.load(nrows=nrows, offset=offset)
@@ -187,15 +191,16 @@ class TestProduct:
         assert layer_query_result.equals(df)
 
 
-    def test_arcgis_legacy_server(self, csvfile, source, last, skip, loghtml):
-        url = "https://opendata.baltimorecity.gov/egis/rest/services/Hosted/911_Calls_For_Service_2017_csv/FeatureServer/0" 
+    # No datasets currently trigger usage of the legacy server code
+    # def test_arcgis_legacy_server(self, csvfile, source, last, skip, loghtml):
+    #     url = "https://egis.baltimorecity.gov/egis/rest/services/GeoSpatialized_Tables/Arrest/FeatureServer/0/" 
 
-        gis = data_loaders.Arcgis(url)
-        gis.load(nrows=1)
+    #     gis = data_loaders.Arcgis(url)
+    #     gis.load(nrows=1)
 
-        url = "https://xmaps.indy.gov/arcgis/rest/services/OpenData/OpenData_NonSpatial/MapServer/5/" 
-        gis = data_loaders.Arcgis(url)
-        gis.load(nrows=1)
+    #     url = "https://xmaps.indy.gov/arcgis/rest/services/OpenData/OpenData_NonSpatial/MapServer/5/" 
+    #     gis = data_loaders.Arcgis(url)
+    #     gis.load(nrows=1)
 
 
     def test_arcgis_geopandas(self, csvfile, source, last, skip, loghtml):
@@ -224,6 +229,11 @@ class TestProduct:
         assert type(df) == pd.DataFrame
         assert len(df) == count
 
+        count2 = gis.get_count(year=year_filter+1)
+
+        # Ensure that count updates properly with different call (most recent count is cached)
+        assert count!=count2
+
     def test_socrata_geopandas(self, csvfile, source, last, skip, loghtml):
         if _has_gpd:
             url = "data.montgomerycountymd.gov"
@@ -244,7 +254,7 @@ class TestProduct:
         date_field = "created_dt"
         year = 2020
         loader = data_loaders.Socrata(url=url, data_set=data_set, date_field=date_field)
-        df = loader.load(year=year)
+        df = loader.load(year=year, pbar=False)
         count = loader.get_count(year=year)
 
         # Reset
@@ -253,23 +263,30 @@ class TestProduct:
         assert type(df) == pd.DataFrame
         assert len(df) == count
 
+        count2 = loader.get_count(year=year+1)
+
+        # Ensure that count updates properly with different call (most recent count is cached)
+        assert count!=count2
+
     def test_socrata(self, csvfile, source, last, skip, loghtml):
         lim = data_loaders._default_limit
         data_loaders._default_limit = 500
         url = "data.austintexas.gov"
         data_set = "sc8s-w4ka"
         loader = data_loaders.Socrata(url, data_set)
-        df =loader.load()
+        df =loader.load(pbar=False)
         count = loader.get_count()
 
+        assert len(df)==count
+
         offset = 1
         nrows = len(df)-offset-1
-        df_offset = loader.load(offset=offset,nrows=nrows)
+        df_offset = loader.load(offset=offset,nrows=nrows, pbar=False)
         assert set(df.columns)==set(df_offset.columns)
         df_offset = df_offset[df.columns]
         assert df_offset.equals(df.iloc[offset:nrows+offset].reset_index(drop=True))
 
-        df_offset = loader.load(offset=offset)
+        df_offset = loader.load(offset=offset, pbar=False)
         assert set(df.columns)==set(df_offset.columns)
         df_offset = df_offset[df.columns]
         assert df_offset.equals(df.iloc[offset:].reset_index(drop=True))
@@ -287,21 +304,24 @@ class TestProduct:
         url = "https://www.denvergov.org/media/gis/DataCatalog/denver_police_officer_involved_shootings/csv/denver_police_officer_involved_shootings.csv"
         date_field = "INCIDENT_DATE"
         loader = data_loaders.Csv(url, date_field=date_field)
-        df = loader.load()
+        df = loader.load(pbar=False)
 
         offset = 1
         nrows = len(df)-offset-1
-        df_offset = loader.load(offset=offset,nrows=nrows)
+        df_offset = loader.load(offset=offset,nrows=nrows, pbar=False)
         assert df_offset.equals(df.iloc[offset:nrows+offset].reset_index(drop=True))
         
-        df_offset = loader.load(offset=offset)
+        df_offset = loader.load(offset=offset, pbar=False)
         assert df_offset.equals(df.iloc[offset:].reset_index(drop=True))
 
         df_comp = pd.read_csv(url)
         df_comp = df_comp.astype({date_field: 'datetime64[ns]'})
+        df = df.astype({date_field: 'datetime64[ns]'})
 
         count = loader.get_count()
         assert len(df_comp) == count
+        # Test using cached value
+        assert count == loader.get_count()
 
         assert df_comp.equals(df)
 
@@ -324,26 +344,31 @@ class TestProduct:
         url = "https://www.denvergov.org/media/gis/DataCatalog/denver_police_officer_involved_shootings/csv/denver_police_officer_involved_shootings.csv"
         loader = data_loaders.Csv(url, date_field="INCIDENT_DATE")
         year = 2020
-        df = loader.load(year=year)
+        df = loader.load(year=year, pbar=False)
         with pytest.raises(ValueError):
             count = loader.get_count(year=year)
 
         count = loader.get_count(year=year, force=True)
         assert len(df) == count
 
+        count2 = loader.get_count(year=year+1, force=True)
+
+        # Ensure that count updates properly with different call (most recent count is cached)
+        assert count!=count2
+
 
     def test_excel(self, csvfile, source, last, skip, loghtml):
         url = "https://www.norristown.org/DocumentCenter/View/1789/2017-2018-Use-of-Force"
         date_field = "Date"
         loader = data_loaders.Excel(url, date_field=date_field)
-        df = loader.load()
+        df = loader.load(pbar=False)
 
         offset = 1
         nrows = len(df)-offset-1
-        df_offset = loader.load(offset=offset,nrows=nrows)
+        df_offset = loader.load(offset=offset,nrows=nrows, pbar=False)
         assert df_offset.equals(df.iloc[offset:nrows+offset].reset_index(drop=True))
 
-        df_offset = loader.load(offset=offset)
+        df_offset = loader.load(offset=offset, pbar=False)
         assert df_offset.equals(df.iloc[offset:].reset_index(drop=True))
 
         df_comp = pd.read_excel(url)
@@ -355,6 +380,9 @@ class TestProduct:
         count = loader.get_count(force=True)
         assert len(df_comp) == count
 
+        # Testing 2nd call which should used cached value
+        assert count == loader.get_count(force=True)
+
         assert df_comp.equals(df)
 
         with pytest.raises(ValueError):
@@ -366,7 +394,7 @@ class TestProduct:
         assert list(df[date_field].dt.year.sort_values(ascending=True).dropna().unique()) == years
 
         nrows = 7
-        df = loader.load(nrows=nrows)        
+        df = loader.load(nrows=nrows, pbar=False)        
         df_comp = pd.read_excel(url, nrows=nrows)
         df_comp = df_comp.convert_dtypes()
         df_comp.columns = [x.strip() if isinstance(x, str) else x for x in df_comp.columns]
@@ -389,7 +417,7 @@ class TestProduct:
         df_comp = df_comp.iloc[:, 1:]
 
         # Load all years
-        df_2014 = loader.load(year=2014)
+        df_2014 = loader.load(year=2014, pbar=False)
 
         assert df_comp.equals(df_2014)
 
@@ -402,17 +430,17 @@ class TestProduct:
         df_comp = df_comp.iloc[:, 1:]
 
         # Load all years
-        df_2015 = loader.load(year=2015)
+        df_2015 = loader.load(year=2015, pbar=False)
 
         assert df_comp.equals(df_2015)
 
         # Note: There is no 2013 data
-        df_multi = loader.load(year=[2013,2015])
+        df_multi = loader.load(year=[2013,2015], pbar=False)
 
         assert df_multi.equals(pd.concat([df_2014, df_2015], ignore_index=True))
 
-        df = loader.load()
-        df_last = loader.load(year=years[-1])
+        df = loader.load(pbar=False)
+        df_last = loader.load(year=years[-1], pbar=False)
 
         assert df.head(len(df_multi)).equals(df_multi)
         assert df.tail(len(df_last)).reset_index(drop=True).equals(df_last.reset_index(drop=True))
@@ -424,7 +452,7 @@ class TestProduct:
         url = "https://cms7files1.revize.com/sparksnv/Document_Center/Sparks%20Police/IA%20Data/2000-2022-SPD-OIS-Incidents%20(3).xlsx"
 
         loader = data_loaders.Excel(url)
-        df = loader.load()
+        df = loader.load(pbar=False)
 
         df_comp = pd.read_excel(url)
         df_comp.columns= [x for x in df_comp.iloc[3]]
@@ -432,6 +460,7 @@ class TestProduct:
         df_comp.reset_index(drop=True, inplace=True)
         df_comp = df_comp.convert_dtypes()
         df_comp.columns = [x.strip() if isinstance(x, str) else x for x in df_comp.columns]
+        df_comp = df_comp.dropna(thresh=10)
 
         assert(df_comp.equals(df))
 
@@ -483,7 +512,7 @@ class TestProduct:
         os.remove(file_path_decrypted)
 
         loader = data_loaders.Excel(url)
-        df = loader.load()
+        df = loader.load(pbar=False)
 
         df_comp = df_comp.convert_dtypes()
         df_comp.columns = [x.strip() if isinstance(x, str) else x for x in df_comp.columns]
@@ -492,19 +521,18 @@ class TestProduct:
 if __name__ == "__main__":
     tp = TestProduct()
 
-    tp.test_arcgis_legacy_server(None,None,None,None,None)
-    tp.test_carto(None,None,None,None,None)
-    tp.test_arcgis(None,None,None,None,None)
-    tp.test_arcgis_geopandas(None,None,None,None,None)
-    tp.test_arcgis_pandas(None,None,None,None,None)
-    tp.test_csv(None,None,None,None,None)
-    tp.test_csv_year_filter(None,None,None,None,None)
-    tp.test_process_date_input_empty(None,None,None,None,None)
-    tp.test_process_date_too_many(None,None,None,None,None)
-    tp.test_process_dates_year_input_wrong_order(None,None,None,None,None)
-    tp.test_socrata(None,None,None,None,None)
-    tp.test_socrata_geopandas(None,None,None,None,None)
-    tp.test_socrata_pandas(None,None,None,None,None)
+    # tp.test_carto(None,None,None,None,None)
+    # tp.test_arcgis(None,None,None,None,None)
+    # tp.test_arcgis_geopandas(None,None,None,None,None)
+    # tp.test_arcgis_pandas(None,None,None,None,None)
+    # tp.test_csv(None,None,None,None,None)
+    # tp.test_csv_year_filter(None,None,None,None,None)
+    # tp.test_process_date_input_empty(None,None,None,None,None)
+    # tp.test_process_date_too_many(None,None,None,None,None)
+    # tp.test_process_dates_year_input_wrong_order(None,None,None,None,None)
+    # tp.test_socrata(None,None,None,None,None)
+    # tp.test_socrata_geopandas(None,None,None,None,None)
+    # tp.test_socrata_pandas(None,None,None,None,None)
     tp.test_excel(None,None,None,None,None)
     tp.test_excel_year_sheets(None,None,None,None,None)
     tp.test_excel_header(None,None,None,None,None)
